/*
 *    TenTecCAT.c  --  TenTecCAT (rig control) interface
 *
 *    Copyright (C) 2001, 2002, 2003, 2004
 *      Tomi Manninen (oh2bns@sral.fi)
 *
 *    This file is part of gMFSK.
 *
 *    gMFSK is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    gMFSK is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with gMFSK; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "hkj-config.h"

#if WANT_TENTEC_MOD

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <pthread.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>

#include "support.h"

#include "TenTecCAT.h"
#include "main.h"
#include "conf.h"
#include "qsodata.h"
#include "trx.h"

/* ---------------------------------------------------------------------- */

//G_LOCK_DEFINE_STATIC(TenTecCAT_mutex);

#define PTT_OFF FALSE
#define PTT_ON  TRUE

static pthread_t TenTecCAT_thread;
static gboolean TenTecCAT_exit = FALSE;
static gboolean TenTecCAT_enabled;
static void *TenTecCAT_loop(void *args);
static gboolean TenTecCAT_PTT = PTT_OFF;
static gboolean TenTec_qsy = FALSE;
static gboolean TogglePTT = FALSE;
static gboolean TenTecCATisPTT = FALSE;

char szFreqFormat[] = "%.6f";

struct ST_SHMEM {
	int  flag;
	long freq;
	long midfreq;
}sharedmem;


struct ST_SHMEM *freqflag;
struct ST_SHMEM noshare;

void *shared_memory = (void *)0;
int  shmid = -1;

key_t hash (char *str)
{
	key_t key = 1;
	int i;
	for (i = 0; i < strlen(str); i++)
		key *= str[i];
	return key;
}

void TenTecCAT_init(void)
{
  szFreqFormat[2] = '0' + TEN_TEC_LOG_PREC;
  
	TenTecCAT_enabled = FALSE;
	noshare.freq = 7070000L;
	noshare.midfreq = 1000L;
	noshare.flag = 2;
	freqflag = &noshare;
//	gboolean fail = FALSE;

	shmid = shmget ((key_t)1234, sizeof(sharedmem), 0666 | IPC_CREAT);

	if (shmid < 0) {
		  errmsg(_("shmget failed"));
		  return;
		 }
 	shared_memory = shmat (shmid, (void *)0, 0);
 	if (shared_memory == (void *)-1) {
 	  errmsg(_("shmat failed"));
 	  return;
 	}
	freqflag = (struct ST_SHMEM *) shared_memory;

	TenTec_qsy = FALSE;
	TenTecCAT_PTT = PTT_OFF;
	TogglePTT = FALSE;
	TenTecCATisPTT = FALSE;
	
	if (pthread_create(&TenTecCAT_thread, NULL, TenTecCAT_loop, NULL) < 0) {
		errmsg(_("TenTecCAT init: pthread_create: %m"));
		return;
	}
	TenTecCAT_enabled = TRUE;
}

void TenTecCAT_close(void)
{
	if (!TenTecCAT_enabled) return;

	/* tell the TenTecCAT thread to kill it self */
	TenTecCAT_exit = TRUE;

	/* and the wait for it to die */
	pthread_join(TenTecCAT_thread, NULL);

	TenTecCAT_exit = FALSE;
}

gboolean TenTecCAT_active(void)
{
	return (TenTecCAT_enabled);
}

gboolean TenTecCAT_CanPTT(void)
{
	return TenTecCATisPTT;
}
	
void setTenTecCAT_PTT (gboolean on)
{
  TenTecCAT_PTT = on;
  TogglePTT = TRUE;
}

void TenTecCAT_set_qsy(void)
{
	if (!TenTecCAT_enabled)
		return;

	TenTec_qsy = TRUE;
}

static void *TenTecCAT_loop(void *args)
{
	gdouble freq = 0.0;
	gboolean sb = TRUE;
	gchar *str;
	struct timespec sleep;

	freqflag->freq = 0L;
	freqflag->flag = -1;
	sb = FALSE;

	for (;;) {
	/* see if we are being canceled */
		if (TenTecCAT_exit)
			break;

	/* delay for 60 msec interval */
		sleep.tv_sec = 0;
		sleep.tv_nsec = 60000000L;	/* 60ms */
		nanosleep(&sleep, NULL);

		if (TenTec_qsy) {
			gdouble f = freqflag->midfreq; //conf_get_double("TenTec/cfreq");
			gdouble f0 = trx_get_freq();
			long corr;

			TenTec_qsy = FALSE;
			corr = (long)(f0 - f);
			freqflag->freq += corr;
			freqflag->flag = -2; // set frequency
			sleep.tv_sec = 0;
			sleep.tv_nsec = 20000000L;	/* 20ms */
			nanosleep(&sleep, NULL);
			f = f0 - corr;
			waterfall_set_frequency(waterfall, f);
		} else if (TogglePTT) {
			if (TenTecCAT_PTT == PTT_ON)
				freqflag->flag = -3;
			else
				freqflag->flag = -4;
			TogglePTT = FALSE;
			sleep.tv_sec = 0;
			sleep.tv_nsec = 20000000L;	/* 20ms */
			nanosleep(&sleep, NULL);
		} else {
	/* delay for 20 msec interval */
			sleep.tv_sec = 0;
			sleep.tv_nsec = 20000000L;	/* 20ms */
			nanosleep(&sleep, NULL);
		}

		if (freqflag->flag >= 0) {
			freqflag->flag = -1; // read frequency
//			sb = FALSE;
		}
	/* delay for 20 msec interval */
		sleep.tv_sec = 0;
		sleep.tv_nsec = 20000000L;	/* 20ms */
		nanosleep(&sleep, NULL);

	/* if TenTecCAT control program running it will update the freqflag structure every 10 msec */
		if (freqflag->flag > -1) {
			freq = freqflag->freq;
			if ((freqflag->flag & 0x10) == 0x10)
				TenTecCATisPTT = TRUE;
			if ((freqflag->flag & 0x0F) == 2)
				sb = TRUE;
			else
				sb = FALSE;
			waterfall_set_carrier_frequency(waterfall, freq);
			waterfall_set_lsb(waterfall, sb);
#if WANT_FREQLOCK
			str = g_strdup_printf(
				szFreqFormat, 
				(sb ? (freq - trx_get_txfreq()) : (freq + trx_get_txfreq())) / 1000000.0);
#else
			str = g_strdup_printf(
				szFreqFormat, 
				(sb ? (freq - trx_get_freq()) : (freq + trx_get_freq())) / 1000000.0);
#endif
			gdk_threads_enter();
				qsodata_set_band_mode(QSODATA_BAND_ENTRY);
				qsodata_set_freq(str);
			gdk_threads_leave();
			g_free(str);
		}
	}

	waterfall_set_carrier_frequency(waterfall, 0.0);
	waterfall_set_lsb(waterfall, FALSE);
	gdk_threads_enter();
			qsodata_set_band_mode(QSODATA_BAND_ENTRY);
			qsodata_set_freq("");
	gdk_threads_leave();

	if (shmdt(shared_memory) == -1) {
		errmsg(_("shmdt failed"));
	}

	shmid = -1;
	TenTecCAT_enabled = FALSE;

	/* this will exit the TenTecCAT thread */
	return NULL;
}

/* ---------------------------------------------------------------------- */

#endif
