/*
 *    ptt.c  --  PTT control
 *
 *    Copyright (C) 2001, 2002, 2003, 2004
 *      Tomi Manninen (oh2bns@sral.fi)
 *
 *    This file is part of gMFSK.
 *
 *    gMFSK is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    gMFSK is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with gMFSK; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>


#include "main.h"
#include "ptt.h"
#include "hamlib.h"
#include "cwirc.h"

#if WANT_TENTEC_MOD
#include "TenTecCAT.h"
#endif

/* ---------------------------------------------------------------------- */

static gint openptt(void);

static gint pttfd = -1;
static gint pttinv = 0;
static gint pttarg = TIOCM_RTS | TIOCM_DTR;
static gchar *pttpath = NULL;
static enum { NOPORT, SERPORT, PARPORT } pttmode = NOPORT;
#if WANT_PTT_MOD
static gint pttDTR = 0;
#endif

struct termios oldtio;
/* ---------------------------------------------------------------------- */

void ptt_init(const gchar *path, gint inverted, gint mode)
{
#if WANT_KACHINA
	return;
#endif

	if (cwirc_extension_mode)
		return;

	/* If it's open then close it */
	ptt_close();

	pttinv = inverted;

#if WANT_PTT_MOD
	pttDTR = 0;
#endif

	switch (mode) {
	case 0:
		pttarg = TIOCM_RTS;
		break;
	case 1:
		pttarg = TIOCM_DTR;
		break;
	case 2:
		pttarg = TIOCM_RTS | TIOCM_DTR;
		break;
#if WANT_PTT_MOD
	case 3:
		pttarg = TIOCM_RTS;
		pttDTR = 1;
		ptt_set(0); // set port for RTS initially off; DTR always on
		break;
#endif
	}

	g_free(pttpath);

	if (!strcasecmp(path, "none"))
		pttpath = NULL;
	else
		pttpath = g_strdup(path);
}

void ptt_close(void)
{
#if WANT_KACHINA
	return;
#endif
	if (pttfd != -1) {
#if WANT_PTT_MOD
		tcsetattr (pttfd, TCSANOW, &oldtio);
#endif
		close(pttfd);

		pttfd = -1;
	}
}

void ptt_set(gint ptt)
{
#if WANT_KACHINA
		setTenTecCAT_PTT (ptt);
		return;
#endif

	if (cwirc_extension_mode)
		return;

#if WANT_HAMLIB
	/* try hamlib ptt first */
	hamlib_set_ptt(ptt);
#endif

	if (!pttpath)
		return;

	/* If it's closed, try to open it */
	if (pttfd < 0 && (pttfd = openptt()) < 0)
		return;

	if (pttinv)
		ptt = !ptt;
	else
		ptt = !!ptt;

	if (pttmode == SERPORT) {
#if WANT_PTT_MOD
		if (!pttDTR) {
			guint arg = pttarg;
			if (ioctl(pttfd, ptt ? TIOCMBIS : TIOCMBIC, &arg) < 0)
				errmsg(_("set_ptt: ioctl: %m"));
		} else {
			guint arg;
			if (ioctl(pttfd, TIOCMGET, &arg) < 0)
			  errmsg(_("get_IOCTL: iotcl: %m"));
			arg |= TIOCM_DTR; /* set the DTR bit */
			if (ptt) /* set or clear the RTS bit */
			  arg |= TIOCM_RTS;
			else
				arg &= ~TIOCM_RTS;
			if (ioctl(pttfd, TIOCMSET, &arg) < 0)
				errmsg(_("set_RTS: ioctl: %m"));
		}
#else
		guint arg = pttarg;
		if (ioctl(pttfd, ptt ? TIOCMBIS : TIOCMBIC, &arg) < 0)
			errmsg(_("set_ptt: ioctl: %m"));
#endif
	}

}

/* ---------------------------------------------------------------------- */

static gint openptt(void)
{
	gint fd;
#if WANT_PTT_MOD
	guint serarg = pttarg;
#else
	guint serarg = TIOCM_RTS | TIOCM_DTR;
#endif

#if WANT_KACHINA
		return 0;
#endif
	if ((fd = open(pttpath, O_RDWR, 0)) < 0) {
		errmsg(_("Cannot open PTT device '%s': %m"), pttpath);
		return -1;
	}
#if WANT_PTT_MOD
	tcgetattr (fd, &oldtio);
#endif

	pttmode = NOPORT;

	if (pttmode == NOPORT && !ioctl(fd, TIOCMBIC, &serarg))
		pttmode = SERPORT;

	if (pttmode == NOPORT) {
		errmsg(_("Device '%s' is not a serial port"), pttpath);
		return -1;
	}

	return fd;
}

/* ---------------------------------------------------------------------- */

